#!/bin/sh
#
# aggiungi_COLOPHON.sh
#
# Aggiunge la sezione COLOPHON alle pagine di manuale.
# Utile per aggiornare il pacchetto man-pages-it a una nuova versione.
# Per sostituire la sezione COLOPHON in tutte le pagine del pacchetto, dalla
# directory man-pages eseguire:
# 
# sh scripts/remove_COLOPHON.sh man?/*
# sh scripts/aggiungi_COLOPHON.sh man?/*

for f in "$@"; do
    sed -i '$ r colophon' "$f"
done
